Package: astronomR
Type: Package
Title: Cosmic Insights: Statistical Frameworks for Astronomers
Version: 0.1.0
Authors@R: c(
    person("Samrit", "Pramanik", email = "samrit.2442@gmail.com", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0003-2055-9007")),
    person("Kazi Abu", "Rousan", email = "kaziaburousan1999@gmail.com", role = c("aut"),
           comment = c(ORCID = "0000-0002-7906-2030")))
Description: A comprehensive toolkit for astronomical and cosmological
    computations. Provides functions for angular coordinate conversions
    (degrees, hours-minutes-seconds, degrees-minutes-seconds, and radians),
    access to fundamental physical constants, queries to the Gaia Archive TAP
    (Table Access Protocol) service, cosmological distance calculations, and
    early-universe thermal
    physics including photon density and 'Saha' equation solutions.
License: MIT + file LICENSE
Encoding: UTF-8
Language: en-US
URL: https://github.com/samrit2442/astronomR
BugReports: https://github.com/samrit2442/astronomR/issues
Imports: dplyr, httr, jsonlite, pracma, readr, stringr, tibble, tidyr
Depends: R (>= 4.1.0)
Suggests: testthat (>= 3.0.0)
Config/testthat/edition: 3
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-05-19 12:08:19 UTC; samri
Author: Samrit Pramanik [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-2055-9007>),
  Kazi Abu Rousan [aut] (ORCID: <https://orcid.org/0000-0002-7906-2030>)
Maintainer: Samrit Pramanik <samrit.2442@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-27 09:20:02 UTC
Built: R 4.5.2; ; 2026-05-27 11:35:16 UTC; unix

# Generated by roxygen2: do not edit by hand

export(Mpc_to_km)
export(Saha_Xe)
export(constant_value)
export(constants_df)
export(deg2rad)
export(deg_to_dms)
export(deg_to_hms)
export(dms_to_deg)
export(get_gaia_data)
export(hms_to_deg)
export(km_to_Mpc)
export(photon_energy_density_fn_T)
export(photon_energy_density_fn_z)
export(photon_number_density_fn_T)
export(photon_number_density_fn_z)
export(rad2deg)
export(soln_saha)

# astronomR: Astronomy and Cosmology Analysis in R 🌌✨

An R package designed to bridge the gap between data science and the cosmos!

## Why {astronomR}?

While R is widely used in data science and statistical computing, there has been a lack of tools specifically tailored for astronomical data. {astronomR} aims to fill this gap, offering easy-to-use functions and tools for anyone looking to explore space and cosmology through data. Whether you’re an astrophysicist or just a data enthusiast, {astronomR} is here to help you navigate the universe.

## Installing this Package

``` r
# Install devtools package if you don't have it
# install.packages("devtools")

devtools::install_github("samrit2442/astronomR")
```

## Introduction

This package was developed for astronomy, cosmological computation, and analysis with R. The source code can be found here <https://github.com/samrit2442/astronomR>

## Some Usage and Example Codes

Normally in astronomy, we use a different sort of angular system for location. We can easily do that in our package. Let’s see how. Suppose, we have an angular value of `d = 177.74208°` We want to convert it into an hour-minute-second. There is a very simple function.

``` r
deg_to_hms(177.74208)
#> 11H50M58.0992S
hms_to_deg(11, 50, 58.09925)
#> [1] 177.7421
```

What else can be done? Let’s say we want to find the path of some star in a particular location for a time interval. This can also be done using our package.

To do that, first, we need to define the RA and Dec value of the star. Let’s see to do this. Also, let’s define the time of observation and its location.

``` r
ra_hour <- 16.695  # RA in hours
dec_deg <- 36.466667  # Dec in degrees
lat_obs <- 52.5  # Observer's latitude
lon_obs <- -1.9166667  # Observer's longitude
datetime <- as.POSIXct("1998-08-10 23:10:00", tz = "UT")  # Observation time
```

Now, create the star location as seen from that location on the time mentioned. This returns Altitude and Azimuth as normally used.

``` r
star_location <- RA_dec2Alt_azi(ra_hour, dec_deg, lat_obs, lon_obs, datetime)
print(paste("Altitude:", star_location$altitude, "degrees"))
#> [1] "Altitude: 49.1688687197424 degrees"

print(paste("Azimuth:", star_location$azimuth, "degrees"))
#> [1] "Azimuth: 269.146669462321 degrees"
```

This tells us from some location, what is the position of the star so that we can use a telescope to watch it! Using this simple function, we can trace out the path any star travels. Let’s see how and also maybe plot it. For that, let’s first make a time range, for which we want to see the location.

``` r
# Plotting Rigel's Motion
start_datetime <- as.POSIXct("2024-10-02 00:00:00", tz = "UTC")
end_datetime <- as.POSIXct("2024-10-03 00:00:00", tz = "UTC")

# Generate timestamps at 20-minute intervals
timestamps <- seq(from = start_datetime, to = end_datetime, by = "20 mins")
altitude <- numeric(length(timestamps))
azimuth <- numeric(length(timestamps))

observer_lat <- 43.1566  # Latitude in degrees
observer_lon <- -77.6088  # Longitude in degrees

# RA and Dec for Rigel (converted RA to hours)
rigel_ra <- 78.634467 / 15  # RA in hours
rigel_dec <- -8.20164  # Dec in degrees

# Calculate altitude and azimuth for each timestamp
for (i in seq_along(timestamps)) {
  datetime <- timestamps[i]
  result <- RA_dec2Alt_azi(rigel_ra, rigel_dec, observer_lat, observer_lon, datetime)
  altitude[i] <- result$altitude
  azimuth[i] <- result$azimuth
}

# Create a data frame with results
rigel_positions <- data.frame(datetime = timestamps, altitude = altitude, azimuth = azimuth)

# 2D Plot for Altitude over Time
library(ggplot2)
ggplot(rigel_positions, aes(x = datetime, y = altitude)) +
  geom_line() +
  labs(x = "Time", y = "Altitude (degrees)", title = "Altitude of Rigel over 24 hours")
```

![image](https://github.com/user-attachments/assets/e7052a09-3142-4796-a505-d60c65a991f7)

``` r
# 2D Plot for Azimuth over Time
ggplot(rigel_positions, aes(x = datetime, y = azimuth)) +
  geom_line() +
  labs(x = "Time", y = "Azimuth (degrees)", title = "Azimuth of Rigel over 24 hours")
```

![image](https://github.com/user-attachments/assets/80dd618d-d7fc-40b3-af08-606ed7d9846c)

``` r
# 2D Scatter Plot for Azimuth vs Altitude
ggplot(rigel_positions, aes(x = azimuth, y = altitude)) +
  geom_point(color = "firebrick") +
  labs(x = "Azimuth (degrees)", y = "Altitude (degrees)", title = "Azimuth vs Altitude for Rigel over 24 hours")
```

![image](https://github.com/user-attachments/assets/46e75c1b-79c8-4163-bdd1-6a2d74311f91)

## Gaia Data Archive

The **Gaia Data Archive** is a comprehensive database that houses the data collected by the European Space Agency’s Gaia mission. Launched in December 2013, Gaia is designed to create the most accurate three-dimensional map of the Milky Way galaxy by observing and cataloguing the positions, distances, and motions of over a billion stars. In Python **Astropy** and **Astroquery** is used to import data directly. We can do the same thing in our package. Let’s see how:

``` r
df <- get_gaia_data(vars = "ra, dec, parallax", condition = "parallax > 50")
head(df)

#>         ra       dec  parallax
#> 1 316.7537  38.75607 286.00534
#> 2 316.7485  38.76386 285.99493
#> 3 298.4819  44.41291 214.57451
#> 4 249.3875 -53.69952  53.22840
#> 5 243.4453 -57.57679  73.54824
#> 6 312.2788  37.47123  56.86465
```

Nice! Isn’t it? Let’s use this for some analysis. Why not create an H-R diagram.

For this first, let’s convert parallax to absolute magnitude.

``` r
library(dplyr)
calculate_absolute_magnitude <- function(parallax, g_mag) {
  distance_pc <- 1 / (parallax / 1000)
  abs_mag <- g_mag - 5 * (log10(distance_pc) - 1)
  return(abs_mag)
}

# Let's import the data
df <- get_gaia_data(vars = "ra, dec, parallax, phot_g_mean_mag, phot_bp_mean_mag, phot_rp_mean_mag", 
                    condition = "parallax > 50")
#data processing
df <- df %>%
  mutate(color_index = phot_bp_mean_mag - phot_rp_mean_mag,
         abs_mag = calculate_absolute_magnitude(parallax, phot_g_mean_mag))
# Filter out any rows with NA values
df <- df %>% filter(!is.na(color_index), !is.na(abs_mag))

# Plot an H-R diagram
ggplot(df, aes(x = color_index, y = abs_mag)) +
  geom_point(alpha = 0.5, color = "blue") +
  labs(x = "Color Index (G_BP - G_RP)", y = "Absolute Magnitude (M)", 
       title = "Hertzsprung-Russell Diagram") +
  theme_minimal() +
  scale_y_reverse()  # Reverse the y-axis for magnitude
```

![image](https://github.com/user-attachments/assets/a29b22c9-9ca0-4048-9bf4-8e50847157c7)

## Cosmological Calculation

We can do many things related to cosmology using this. Let’s see a few of them. Before that, I will suggest to run this code. As we know, there are many cosmological models. You can define these in our package.

``` r
cosmo <- FlatLCDM(0.6774, 0.6911, 0.3089, 8.4e-5)
cosmo
#> $hubble_constant_fact
#> [1] 0.6774
#> 
#> $dark_matter_crit
#> [1] 0.69110000000000005
#> 
#> $matter_crit
#> [1] 0.30890000000000001
#> 
#> $radiation_crit
#> [1] 8.3999999999999995e-05
#> 
#> $type
#> [1] "FlatLCDM"
#> 
#> $h_per_s
#> [1] 2.1949477836373805e-20
```

Now, we can directly use these models and inputs of other functions. As we know scale factor of our universe `a = 1` today. We can directly use our function to calculate time at any given `a` value.

``` r
t_as_func_a_in_Year(cosmo, 1) # in year
#> [1] 13808979942.748983
```

We can find the age of our universe using this or use the function `age_of_universe_GY()`.

``` r
age_of_universe_GY(cosmo) # our universe age !!!!!
#> [1] 13.808979942748984
```

Our package can find the radius of curvature of our universe. Let’s see:

``` r
radius_of_curvature(cosmo)
#> [1] 0
```

#### *Many more functions are coming soon. Wait for the documentation for more!*

julian_day <- function(datetime) {
  year <- as.numeric(format(datetime, "%Y"))
  month <- as.numeric(format(datetime, "%m"))
  day <- as.numeric(format(datetime, "%d"))
  hour <- as.numeric(format(datetime, "%H")) + as.numeric(format(datetime, "%M")) / 60

  if (month <= 2) {
    year <- year - 1
    month <- month + 12
  }

  A <- floor(year / 100)
  B <- 2 - A + floor(A / 4)

  jd <- floor(365.25 * (year + 4716)) + floor(30.6001 * (month + 1)) + day + B - 1524.5 + (hour / 24)

  return(jd)
}


RA_dec2Alt_azi <- function(ra, dec, lat, lon, datetime) {
  jd <- julian_day(datetime)
  # Calculate sidereal time at Greenwich
  gst <- 18.697374558 + 24.06570982441908 * (jd - 2451545.0)
  gst <- gst %% 24 # Convert to 0-24 hour range
  # Convert GST to LST
  lst_hours <- gst + lon / 15.0
  ra_deg <- ra * 15.0
  ha_deg <- lst_hours * 15.0 - ra_deg
  ha_rad <- deg2rad(ha_deg)
  dec_rad <- deg2rad(dec)
  lat_rad <- deg2rad(lat)
  sin_alt <- sin(dec_rad) * sin(lat_rad) + cos(dec_rad) * cos(lat_rad) * cos(ha_rad)
  alt_rad <- asin(sin_alt)

  cos_az <- (sin(dec_rad) - sin(alt_rad) * sin(lat_rad)) / (cos(alt_rad) * cos(lat_rad))
  az_rad <- acos(cos_az)
  if (sin(ha_rad) > 0) {
    az_rad <- 2 * pi - az_rad
  }
  alt_deg <- rad2deg(alt_rad)
  az_deg <- rad2deg(az_rad)
  return(list(altitude = alt_deg, azimuth = az_deg))
}

#' Fundamental Physical Constants in SI and Natural Units
#'
#' A dataset containing commonly used physical constants in both SI units and
#' natural units. The dataset includes the constant's name, symbol, value in
#' SI units, SI unit, value in natural units, and natural unit representation.
#'
#' @format A tibble with 19 rows and 6 columns:
#' \describe{
#'   \item{name}{Character. Name of the physical constant.}
#'   \item{symbol}{Character. Symbol representing the constant.}
#'   \item{value_SI}{Numeric. The value of the constant in SI units.}
#'   \item{unit_SI}{Character. The SI unit for the constant.}
#'   \item{value_Natural}{Numeric. The value of the constant in natural units.}
#'   \item{unit_Natural}{Character. The unit of the constant in natural units.}
#' }
#' @value A \code{tibble} (data frame) with 19 rows and 6 columns:
#' \describe{
#'   \item{name}{Character. Full name of the physical constant.}
#'   \item{symbol}{Character. Symbol representing the constant.}
#'   \item{value_SI}{Numeric. Value of the constant in SI units.}
#'   \item{unit_SI}{Character. The SI unit string for the constant.}
#'   \item{value_Natural}{Numeric. Value of the constant in natural units.}
#'   \item{unit_Natural}{Character. The natural unit string for the constant.}
#' }
#' @examples
#' constants_df
#' @export
constants_df <- tibble::tribble(
  ~name,                      ~symbol,    ~value_SI,        ~unit_SI,  ~value_Natural, ~unit_Natural,
  "Speed of Light",           "c0",       299792458,        "m/s",                1, "-",
  "Magnetic Constant",        "mu0",      4e-7 * pi,        "N A^(-2)",           1, "-",
  "Electric Constant",        "epsilon0", 8.854187817e-12,  "F/m",                1, "-",
  "Gravitational Constant",   "G",        6.67408e-11,      "m^3 kg^(-1) s^(-2)", 1, "-",
  "Planck Constant",          "h",        6.626070040e-34,  "J s",                1, "-",
  "Elementary Charge",        "e",        1.6021766208e-19, "C",                  1, "-",
  "Magnetic Flux Quantum",    "phi0",     2.067833831e-15,  "Wb",                 1, "-",
  "Conductance Quantum",      "G0",       7.7480917310e-5,  "S",                  1, "-",
  "Electron Mass",            "m_e",      9.10938356e-31,   "kg",                 5.10999e5, "eV",
  "Proton Mass",              "m_p",      1.672621898e-27,  "kg",                 1, "-",
  "Fine-Structure Constant",  "alpha",    7.2973525664e-3,  "-",                  1, "-",
  "Rydberg Constant",         "R_inf",    10973731.568508,  "m^(-1)",             1, "-",
  "Avogadro Constant",        "N_A",      6.022140857e23,   "mol^(-1)",           1, "-",
  "Faraday Constant",         "F",        96485.33289,      "C mol^(-1)",         1, "-",
  "Molar Gas Constant",       "R",        8.3144598,        "J mol^(-1) K^(-1)",  1, "-",
  "Boltzman Constant",        "k",        1.38064852e-23,   "J K^(-1)",           1, "-",
  "Stefan-Boltzman Constant", "sigma",    5.670367e-8,      "W m^(-2) K^(-4)",    1, "-",
  "Electron Volt",            "eV",       1.6021766208e-19, "J",                  1, "-",
  "Atomic Mass Unit",         "u",        1.660539040e-27,  "kg",                 1, "-"
)


#' Retrieve the Value of a Physical Constant
#'
#' Retrieves the value and unit of a specified physical constant from the
#' \code{constants_df} dataset. You can search by the constant's name and
#' choose whether to retrieve the value in SI units or natural units.
#'
#' @param constant_name Character. The name (or part of the name) of the
#'   constant to search for. Case-insensitive and partial matches are allowed.
#' @param unit Character. The unit system to retrieve the constant in. Options
#'   are:
#'   \itemize{
#'     \item \code{"SI"}: Retrieves the value in SI units (default).
#'     \item \code{"Natural"}: Retrieves the value in natural units.
#'   }
#' @return A list containing:
#'   \itemize{
#'     \item \code{name}: The full name of the constant.
#'     \item \code{value}: The numeric value of the constant.
#'     \item \code{unit}: The unit string for the selected unit system.
#'   }
#' @examples
#' constant_value("speed of light", unit = "SI")
#' constant_value("planck", unit = "SI")
#' constant_value("electron mass", unit = "Natural")
#' @export
constant_value <- function(constant_name, unit = "SI") {
  matching_rows <- constants_df[
    stringr::str_detect(tolower(constants_df$name), tolower(constant_name)),
  ]

  if (nrow(matching_rows) == 1) {
    constant_row <- matching_rows
  } else if (nrow(matching_rows) == 0) {
    stop("No constant found matching that name.")
  } else {
    stop("Multiple constants match that name. Please specify further.")
  }

  if (unit == "SI") {
    value <- constant_row$value_SI
    unit_value <- constant_row$unit_SI
  } else if (unit == "Natural") {
    value <- constant_row$value_Natural
    unit_value <- constant_row$unit_Natural
  } else {
    stop("Invalid unit. Use 'SI' or 'Natural'.")
  }

  list(
    name = constant_row$name,
    value = value,
    unit = unit_value
  )
}

test_that("constants_df has expected dimensions and columns", {
  expect_equal(nrow(constants_df), 19)
  expect_named(constants_df, c("name", "symbol", "value_SI", "unit_SI",
                               "value_Natural", "unit_Natural"))
})

test_that("constant_value returns correct SI value for speed of light", {
  result <- constant_value("speed of light", unit = "SI")
  expect_equal(result$value, 299792458)
  expect_equal(result$unit, "m/s")
})

test_that("constant_value returns correct Natural value for electron mass", {
  result <- constant_value("electron mass", unit = "Natural")
  expect_equal(result$value, 5.10999e5)
  expect_equal(result$unit, "eV")
})

test_that("constant_value errors on unknown constant name", {
  expect_error(constant_value("dark matter"))
})

test_that("constant_value errors on ambiguous name", {
  expect_error(constant_value("constant"))
})

test_that("constant_value errors on invalid unit", {
  expect_error(constant_value("planck", unit = "CGS"))
})

zeta_3 <- 1.20205690316
EI <- 13.6         # Hydrogen ionization energy (eV)
Me <- 5.10999e5    # Electron mass (eV)
baryon_to_photon_ratio <- 6e-10
nb <- 0.76 * baryon_to_photon_ratio * (2 * zeta_3) / (pi^2)

# Convert redshift to temperature in eV
z_to_temp_eV <- function(z) {
  z0 <- 2.725 / (1.160451812e4)
  z0 * (1 + z)
}

pre_fac_saha <- nb * (2 * pi / Me)^(3 / 2)


#' Photon Energy Density as a Function of Temperature
#'
#' Calculates the photon energy density for a given temperature \eqn{T},
#' using \eqn{\rho_\gamma = \frac{\pi^2}{15} T^4}.
#'
#' @param temp Numeric. Temperature in either eV or Kelvin.
#' @param unit Character. The unit of the temperature input. Either
#'   \code{"eV"} (default) or \code{"K"}.
#' @return Numeric. The photon energy density in natural units.
#' @examples
#' photon_energy_density_fn_T(1, "eV")
#' photon_energy_density_fn_T(300, "K")
#' @export
photon_energy_density_fn_T <- function(temp, unit = "eV") {
  if (unit == "eV") {
    (pi^2 / 15) * temp^4
  } else if (unit == "K") {
    (pi^2 / 15) * (8.6173e-5 * temp)^4
  } else {
    stop("'unit' must be either 'eV' or 'K'.")
  }
}


#' Photon Number Density as a Function of Temperature
#'
#' Calculates the photon number density for a given temperature \eqn{T},
#' using \eqn{n_\gamma = \frac{2\zeta(3)}{\pi^2} T^3}.
#'
#' @param temp Numeric. Temperature in either eV or Kelvin.
#' @param unit Character. The unit of the temperature input. Either
#'   \code{"eV"} (default) or \code{"K"}.
#' @return Numeric. The photon number density in natural units.
#' @examples
#' photon_number_density_fn_T(1, "eV")
#' photon_number_density_fn_T(300, "K")
#' @export
photon_number_density_fn_T <- function(temp, unit = "eV") {
  if (unit == "eV") {
    (2 * zeta_3 / pi^2) * temp^3
  } else if (unit == "K") {
    (2 * zeta_3 / pi^2) * (8.6173e-5 * temp)^3
  } else {
    stop("'unit' must be either 'eV' or 'K'.")
  }
}


#' Photon Energy Density as a Function of Redshift
#'
#' Calculates the photon energy density at a given cosmological redshift
#' \eqn{z} by first converting to temperature in eV.
#'
#' @param z Numeric. The redshift value.
#' @return Numeric. The photon energy density in natural units.
#' @examples
#' photon_energy_density_fn_z(1300)
#' @export
photon_energy_density_fn_z <- function(z) {
  (pi^2 / 15) * z_to_temp_eV(z)^4
}


#' Photon Number Density as a Function of Redshift
#'
#' Calculates the photon number density at a given cosmological redshift
#' \eqn{z} by first converting to temperature in eV.
#'
#' @param z Numeric. The redshift value.
#' @return Numeric. The photon number density in natural units.
#' @examples
#' photon_number_density_fn_z(1300)
#' @export
photon_number_density_fn_z <- function(z) {
  (2 * zeta_3 / pi^2) * z_to_temp_eV(z)^3
}


#' Ionization Fraction Using the Saha Equation
#'
#' Calculates the ionization fraction \eqn{X_e} as a function of redshift
#' \eqn{z} using the Saha equation for hydrogen recombination.
#'
#' @param z Numeric. The redshift value.
#' @return Numeric. The ionization fraction \eqn{X_e} (between 0 and 1).
#' @examples
#' Saha_Xe(1300)
#' Saha_Xe(1100)
#' @export
Saha_Xe <- function(z) {
  temp <- z_to_temp_eV(z)
  f <- pre_fac_saha * temp^(3 / 2) * exp(EI / temp)
  (-1 + sqrt(1 + 4 * f)) / (2 * f)
}


#' Deviation of Ionization Fraction from 0.5
#'
#' Returns \eqn{X_e(z) - 0.5}, useful for finding the redshift at which the
#' ionization fraction equals 0.5 (e.g., via root-finding).
#'
#' @param z Numeric. The redshift value.
#' @return Numeric. \eqn{X_e - 0.5}.
#' @examples
#' soln_saha(1350)
#' @export
soln_saha <- function(z) {
  Saha_Xe(z) - 0.5
}

test_that("photon_energy_density_fn_T is positive for positive T", {
  expect_gt(photon_energy_density_fn_T(1, "eV"), 0)
  expect_gt(photon_energy_density_fn_T(300, "K"), 0)
})

test_that("photon_number_density_fn_T is positive for positive T", {
  expect_gt(photon_number_density_fn_T(1, "eV"), 0)
  expect_gt(photon_number_density_fn_T(300, "K"), 0)
})

test_that("photon_energy_density_fn_T errors on invalid unit", {
  expect_error(photon_energy_density_fn_T(1, "J"))
})

test_that("photon_number_density_fn_T errors on invalid unit", {
  expect_error(photon_number_density_fn_T(1, "J"))
})

test_that("photon_energy_density_fn_z is positive for positive redshift", {
  expect_gt(photon_energy_density_fn_z(1300), 0)
})

test_that("photon_number_density_fn_z is positive for positive redshift", {
  expect_gt(photon_number_density_fn_z(1300), 0)
})

test_that("Saha_Xe returns value between 0 and 1", {
  xe <- Saha_Xe(1300)
  expect_gte(xe, 0)
  expect_lte(xe, 1)
})

test_that("soln_saha returns approximately zero near recombination redshift", {
  # Recombination happens around z ~ 1100-1200
  expect_lt(abs(soln_saha(1150)), 0.5)
})

#' Query Gaia Archive Data
#'
#' Queries the Gaia Archive TAP service to retrieve stellar data based on
#' specified variables and filter conditions. Uses the Gaia Early Data Release
#' 3 (EDR3) catalog.
#'
#' @param vars A character string specifying the variables (columns) to
#'   retrieve, separated by commas (e.g., \code{"source_id, ra, dec"}).
#' @param condition A character string specifying the SQL WHERE clause used to
#'   filter rows (e.g., \code{"parallax > 10"}).
#' @return A data frame containing the requested columns for all rows matching
#'   \code{condition}.
#' @details This function sends a synchronous ADQL query to the Gaia Archive
#'   TAP service at \url{https://gea.esac.esa.int/tap-server/tap/sync}. An
#'   internet connection is required.
#' @examples
#' \donttest{
#' vars <- "source_id, ra, dec, phot_bp_mean_mag, phot_rp_mean_mag, parallax"
#' condition <- "parallax > 40"
#' result <- get_gaia_data(vars, condition)
#' head(result)
#' }
#' @export
get_gaia_data <- function(vars, condition) {
  base_url <- "https://gea.esac.esa.int/tap-server/tap/sync"

  query <- paste0(
    "SELECT ",
    vars,
    " FROM external.gaiaedr3_gcns_main_1 ",
    "WHERE ",
    condition
  )

  body <- list(
    REQUEST = "doQuery",
    LANG = "ADQL",
    FORMAT = "json",
    PHASE = "RUN",
    QUERY = query
  )

  response <- tryCatch(
    httr::POST(base_url, body = body, encode = "form"),
    error = function(e) {
      stop("Failed to connect to Gaia Archive: ", e$message)
    }
  )

  if (httr::status_code(response) != 200) {
    stop(
      "Failed to query Gaia Archive. HTTP status code: ",
      httr::status_code(response)
    )
  }

  content <- httr::content(response, "text", encoding = "UTF-8")
  data <- tryCatch(
    jsonlite::fromJSON(content, flatten = TRUE),
    error = function(e) {
      stop("Failed to parse JSON response from Gaia Archive.")
    }
  )

  if (length(data$data) == 0) {
    stop("No results found for the query.")
  }

  df <- as.data.frame(data$data)
  colnames(df) <- stringr::str_split_1(vars, ",") |>
    stringr::str_squish()

  df
}

#' Convert Decimal Degrees to Degrees, Minutes, and Seconds (DMS) Format
#'
#' Converts a decimal degree angle to the Degrees-Minutes-Seconds (DMS) format
#' commonly used in astronomy for expressing declination and other angles.
#'
#' @param deg Numeric vector of angles in decimal degrees. All values must be
#'   between -90 and 90.
#' @param type Character string specifying the output format. Options are:
#'   \itemize{
#'     \item \code{"cat"} (default): Prints the angle as a formatted string
#'       in DMS notation.
#'     \item \code{"mat"}: Returns a matrix with columns for sign, degrees,
#'       minutes, and seconds.
#'   }
#' @param digit Integer specifying the number of digits to round the seconds
#'   to. Default is \code{5}.
#' @return When \code{type = "cat"}, prints the DMS string and returns
#'   \code{NULL} invisibly. When \code{type = "mat"}, returns a character
#'   matrix with columns \code{SIGN}, \code{DEG}, \code{MIN}, and \code{SEC}.
#' @examples
#' deg_to_dms(45.5042)
#' deg_to_dms(-12.5, type = "mat")
#' deg_to_dms(c(10.25, 45.5), type = "mat")
#' @export
deg_to_dms <- function(deg, type = "cat", digit = 5) {
  if (any(deg < -90 | deg > 90)) {
    stop("All degree values should be between -90\u00B0 and 90\u00B0.")
  }

  df <- tibble::tibble(deg) |>
    dplyr::mutate(
      deg_sign = base::sign(deg),
      deg = abs(deg),
      DEG = floor(deg),
      MIN = floor((deg - DEG) * 60),
      SEC = (deg - DEG - MIN / 60) * 3600,
      SEC = dplyr::case_when(SEC <= 0 ~ 0, SEC > 60 ~ 60, .default = SEC),
      MIN = dplyr::case_when(SEC == 60 ~ MIN + 1, MIN == 60 ~ 0, .default = MIN),
      DEG = dplyr::case_when(MIN == 60 ~ DEG + 1, .default = DEG),
      SEC = round(SEC, digits = digit),
      SIGN = dplyr::if_else(deg_sign == -1, "-", "+"),
      output2 = paste0(SIGN, DEG, "\u00B0", MIN, "'", SEC, '"')
    )

  if (type == "cat") {
    cat(df$output2)
    return(invisible(NULL))
  }
  if (type == "mat") {
    return(df |> dplyr::select(SIGN, DEG, MIN, SEC) |> as.matrix())
  }
}


#' Convert Degrees, Minutes, and Seconds (DMS) to Decimal Degrees
#'
#' Converts an angle expressed in Degrees-Minutes-Seconds (DMS) format to
#' decimal degrees. Accepts either separate numeric arguments or a single
#' formatted string.
#'
#' @param d Numeric or character. The degrees component. If a single character
#'   string is provided (e.g., \code{"12\u00B034'56\""}), the degrees, minutes,
#'   and seconds are parsed automatically, and \code{m} and \code{s} should be
#'   omitted.
#' @param m Numeric. The minutes component. Must be less than 60. Optional when
#'   \code{d} is a string.
#' @param s Numeric. The seconds component. Must be less than 60. Optional when
#'   \code{d} is a string.
#' @param digit Integer. Number of decimal places to round the result to.
#'   Default is \code{5}.
#' @return Numeric. The angle in decimal degrees.
#' @examples
#' dms_to_deg(d = 12, m = 34, s = 56)
#' dms_to_deg(d = "12\u00B034'56\"", digit = 3)
#' @export
dms_to_deg <- function(d, m, s, digit = 5) {
  if (is.character(d) && missing(m) && missing(s)) {
    d2 <- readr::parse_number(d)
    m2 <- stringr::str_remove(d, as.character(d2))
    m <- readr::parse_number(m2)
    s2 <- stringr::str_remove(m2, as.character(m))
    s <- readr::parse_number(s2)
    d <- d2
  }

  if (d < -90 | d > 90) {
    stop("'d' must be between -90 and 90.")
  }
  if (m >= 60 | s >= 60) {
    stop("'m' and 's' must each be less than 60.")
  }

  df <- tibble::tibble(d, m, s) |>
    dplyr::mutate(
      sign = base::sign(d),
      deg = abs(d) + (m / 60) + (s / 3600),
      deg = round(deg, digit) * sign
    )

  df |> dplyr::pull(deg)
}


#' Convert Decimal Degrees to Hours, Minutes, and Seconds (HMS) Format
#'
#' Converts a decimal degree angle to Hours-Minutes-Seconds (HMS) format,
#' which is used in astronomy to express Right Ascension (RA).
#'
#' @param deg Numeric vector of angles in decimal degrees. Values can range
#'   from 0 to 360.
#' @param type Character string specifying the output format. Options are:
#'   \itemize{
#'     \item \code{"cat"} (default): Prints a formatted string
#'       (e.g., \code{3H0M0S}).
#'     \item \code{"mat"}: Returns a matrix with columns for hours, minutes,
#'       and seconds.
#'   }
#' @param digit Integer specifying the number of decimal places to round
#'   seconds to. Default is \code{5}.
#' @return When \code{type = "cat"}, prints the HMS string and returns
#'   \code{NULL} invisibly. When \code{type = "mat"}, returns a numeric matrix
#'   with columns \code{HRS}, \code{MIN}, and \code{SEC}.
#' @examples
#' deg_to_hms(deg = 45)
#' deg_to_hms(deg = 45, type = "mat")
#' deg_to_hms(deg = 177.74208, digit = 3)
#' @export
deg_to_hms <- function(deg, type = "cat", digit = 5) {
  df <- tibble::tibble(deg) |>
    dplyr::mutate(
      DEG = dplyr::if_else(deg < 0, deg + 360, deg),
      HRS = floor(DEG / 15),
      MIN = floor((DEG / 15 - HRS) * 60),
      SEC = (DEG / 15 - HRS - MIN / 60) * 3600,
      SEC = dplyr::case_when(SEC <= 0 ~ 0, SEC > 60 ~ 60, .default = SEC),
      MIN = dplyr::case_when(SEC == 60 ~ MIN + 1, MIN == 60 ~ 0, .default = MIN),
      HRS = dplyr::case_when(MIN == 60 ~ HRS + 1, .default = HRS),
      HRS = HRS %% 24,
      SEC = round(SEC, digits = digit),
      output2 = paste0(HRS, "H", MIN, "M", SEC, "S")
    )

  if (type == "cat") {
    cat(df$output2)
    return(invisible(NULL))
  }
  if (type == "mat") {
    return(df |> dplyr::select(HRS, MIN, SEC) |> as.matrix())
  }
}


#' Convert Hours, Minutes, and Seconds (HMS) to Decimal Degrees
#'
#' Converts an angle expressed in Hours-Minutes-Seconds (HMS) format to
#' decimal degrees. Accepts either separate numeric arguments or a single
#' formatted string.
#'
#' @param h Numeric or character. The hours component. If a single character
#'   string is provided (e.g., \code{"03h15m30s"}), the hours, minutes, and
#'   seconds are parsed automatically, and \code{m} and \code{s} should be
#'   omitted.
#' @param m Numeric. The minutes component. Optional when \code{h} is a
#'   string.
#' @param s Numeric. The seconds component. Optional when \code{h} is a
#'   string.
#' @param digit Integer. Number of decimal places to round the result to.
#'   Default is \code{5}.
#' @return Numeric. The angle in decimal degrees.
#' @examples
#' hms_to_deg(h = 3, m = 15, s = 30)
#' hms_to_deg(h = 3, m = 15, s = 30.123)
#' hms_to_deg(h = "03h15m30s")
#' @export
hms_to_deg <- function(h, m, s, digit = 5) {
  if (is.character(h) && missing(m) && missing(s)) {
    df <- tibble::tibble(h) |>
      dplyr::mutate(h = tolower(h)) |>
      tidyr::separate_wider_regex(
        h,
        patterns = c(H = "[0-9]+", "h", M = "[0-9]+", "m", S = "[0-9.]+", "s"),
        too_few = "align_start"
      ) |>
      dplyr::mutate(
        dplyr::across(c(H, M, S), as.numeric),
        DEG = round((H * 15) + (M * 15 / 60) + (S * 15 / 3600), digits = digit)
      )
  } else {
    df <- tibble::tibble(H = h, M = m, S = s) |>
      dplyr::mutate(DEG = round((H * 15) + (M * 15 / 60) + (S * 15 / 3600),
                                digits = digit))
  }

  df |> dplyr::pull(DEG)
}


#' Convert Degrees to Radians
#'
#' Converts angles from degrees to radians. Supports vectorized input.
#'
#' @param deg Numeric. The angle(s) in degrees to convert to radians.
#' @return Numeric. The corresponding angle(s) in radians.
#' @examples
#' deg2rad(180)
#' deg2rad(c(0, 90, 180))
#' @export
deg2rad <- function(deg) {
  deg * pi / 180
}


#' Convert Radians to Degrees
#'
#' Converts angles from radians to degrees. Supports vectorized input.
#'
#' @param rad Numeric. The angle(s) in radians to convert to degrees.
#' @return Numeric. The corresponding angle(s) in degrees.
#' @examples
#' rad2deg(pi)
#' rad2deg(c(0, pi / 2, pi))
#' @export
rad2deg <- function(rad) {
  rad * 180 / pi
}

test_that("deg2rad and rad2deg are inverses", {
  expect_equal(deg2rad(180), pi)
  expect_equal(deg2rad(0), 0)
  expect_equal(rad2deg(pi), 180)
  expect_equal(rad2deg(0), 0)
  expect_equal(rad2deg(deg2rad(45)), 45)
})

test_that("deg2rad is vectorized", {
  expect_equal(deg2rad(c(0, 90, 180)), c(0, pi / 2, pi))
})

test_that("hms_to_deg gives correct result with numeric inputs", {
  expect_equal(hms_to_deg(h = 3, m = 15, s = 30), 48.875)
  expect_equal(hms_to_deg(h = 0, m = 0, s = 0), 0)
})

test_that("hms_to_deg parses string input", {
  expect_equal(hms_to_deg(h = "03h15m30s"), 48.875)
})

test_that("dms_to_deg gives correct result with numeric inputs", {
  expect_equal(dms_to_deg(d = 12, m = 34, s = 56), 12.58222)
  expect_equal(dms_to_deg(d = 0, m = 0, s = 0), 0)
})

test_that("dms_to_deg errors on out-of-range inputs", {
  expect_error(dms_to_deg(d = 91, m = 0, s = 0))
  expect_error(dms_to_deg(d = 0, m = 60, s = 0))
})

test_that("deg_to_dms errors on out-of-range input", {
  expect_error(deg_to_dms(91))
  expect_error(deg_to_dms(-91))
})

test_that("deg_to_dms mat output has correct structure", {
  result <- deg_to_dms(45.5, type = "mat")
  expect_true(is.matrix(result))
  expect_equal(ncol(result), 4)
  expect_equal(colnames(result), c("SIGN", "DEG", "MIN", "SEC"))
})

test_that("deg_to_hms mat output has correct structure", {
  result <- deg_to_hms(45, type = "mat")
  expect_true(is.matrix(result))
  expect_equal(ncol(result), 3)
  expect_equal(colnames(result), c("HRS", "MIN", "SEC"))
})

# Suppress R CMD check NOTEs for dplyr NSE column name references
utils::globalVariables(c(
  # deg_to_dms
  "DEG", "MIN", "SEC", "deg_sign", "SIGN",
  # deg_to_hms
  "HRS",
  # dms_to_deg
  "deg",
  # hms_to_deg
  "H", "M", "S"
))

#' Convert Kilometers to Megaparsecs
#'
#' This function converts a distance value from kilometers (km) to megaparsecs (Mpc).
#' The conversion factor is based on 1 parsec being equivalent to 3.262 light-years,
#' and 1 light-year being approximately \eqn{9.461 \times 10^{12}} kilometers.
#'
#' @param km A numeric value representing the distance in kilometers.
#'
#' @return A numeric value representing the equivalent distance in megaparsecs (Mpc).
#' @export
#'
#' @examples
#' km_to_Mpc(3.086e19) # Converts 3.086e19 km (approx. 1 Mpc) to megaparsecs
km_to_Mpc <- function(km) {
  km / (3.262e6 * 9.461e12)
}

#' Convert Megaparsecs to Kilometers
#'
#' This function converts a distance value from megaparsecs (Mpc) to kilometers (km).
#' The conversion factor is based on 1 parsec being equivalent to 3.262 light-years,
#' and 1 light-year being approximately \eqn{9.461 \times 10^{12}} kilometers.
#'
#' @param mpc A numeric value representing the distance in megaparsecs.
#'
#' @return A numeric value representing the equivalent distance in kilometers (km).
#' @export
#'
#' @examples
#' Mpc_to_km(1) # Converts 1 Mpc to kilometers
Mpc_to_km <- function(mpc) {
  mpc * 3.262e6 * 9.461e12
}

sec_to_year <- function(sec_ye) {
  3.1709791984e-8 * sec_ye
}

FlatLCDM <- function(hubble_constant_fact, dark_matter_crit, matter_crit, radiation_crit, T_cmb = 2.725) {
  h_per_s <- hubble_constant_fact * 1 / Mpc_to_km(1.0)
  return(list(hubble_constant_fact = hubble_constant_fact, dark_matter_crit = dark_matter_crit, matter_crit = matter_crit, radiation_crit = radiation_crit, type = "FlatLCDM", h_per_s = h_per_s, T_cmb = T_cmb))
}


ClosedLCDM <- function(hubble_constant_fact, curvature_crit, dark_matter_crit, matter_crit, radiation_crit, T_cmb = 2.725) {
  h_per_s <- hubble_constant_fact * 1 / Mpc_to_km(1.0)
  return(list(hubble_constant_fact = hubble_constant_fact, curvature_crit = curvature_crit, dark_matter_crit = dark_matter_crit, matter_crit = matter_crit, radiation_crit = radiation_crit, type = "ClosedLCDM", h_per_s = h_per_s, T_cmb = 2.725))
}


OpenLCDM <- function(hubble_constant_fact, curvature_crit, dark_matter_crit, matter_crit, radiation_crit, T_cmb = 2.725) {
  h_per_s <- hubble_constant_fact * 1 / Mpc_to_km(1.0)
  return(list(hubble_constant_fact = hubble_constant_fact, curvature_crit = curvature_crit, dark_matter_crit = dark_matter_crit, matter_crit = matter_crit, radiation_crit = radiation_crit, type = "OpenLCDM", h_per_s = h_per_s, T_cmb = 2.725))
}

### Only export the function that the user should interact with

cosmology_model <- function(hubble_constant_fact = 0.6774, curvature_crit = 0, dark_matter_crit = 0.6911, matter_crit = 0.3089, radiation_crit = 0, T_cmb = 2.725) {
  if (curvature_crit < 0) {
    return(ClosedLCDM(hubble_constant_fact, curvature_crit, dark_matter_crit, matter_crit, radiation_crit, T_cmb))
  } else if (curvature_crit > 0) {
    return(OpenLCDM(hubble_constant_fact, curvature_crit, dark_matter_crit, matter_crit, radiation_crit, T_cmb))
  } else {
    return(FlatLCDM(hubble_constant_fact, dark_matter_crit, matter_crit, radiation_crit, T_cmb))
  }
}

a <- function(z) {
  1 / (1 + z)
} # a= expansion factor; today it's 1
redshift <- function(a) {
  if (a < 0) {
    stop("Scale factor 'a' cannot be negative.")
  }
  1 / a - 1
}

H_by_H0 <- function(c, a) {
  if (c$type == "FlatLCDM") {
    return(sqrt(c$radiation_crit * a^(-4) + c$matter_crit * a^(-3) + c$dark_matter_crit))
  } else {
    return(sqrt(c$radiation_crit * a^(-4) + c$matter_crit * a^(-3) + c$curvature_crit * a^(-2) + c$dark_matter_crit))
  }
}

tH0_as_fn_a <- function(c, a) {
  helper_fun <- function(a_b) {
    if (c$type == "FlatLCDM") {
      return(a_b / (sqrt(c$radiation_crit + c$matter_crit * a_b + c$dark_matter_crit * a_b^4)))
    } else {
      return(a_b / (sqrt(c$radiation_crit + c$matter_crit * a_b + c$curvature_crit * a_b^2 + c$dark_matter_crit * a_b^4)))
    }
  }
  return(pracma::quad(helper_fun, 0, a))
}

t_as_func_a_in_Year <- function(c, a) {
  k <- (100.0 * c$h_per_s) / (sec_to_year(1.0))
  return(tH0_as_fn_a(c, a) / k)
}


age_of_universe <- function(c, unit = "year") {
  age <- t_as_func_a_in_Year(c, 1.0)

  if (unit == "year") {
    return(age)
  } else if (unit == "GY") {
    return(age / 10^9)
  } else {
    stop("'unit' must be either 'year' or 'GY'.")
  }
}

comv_dist <- function(c) {
  return(2997.92458 / c$h_per_s) # in km
}

comoving_distance <- function(c, z) {
  help_funct <- function(z_1) {
    return(1 / H_by_H0(c, a(z_1)))
  }
  return(pracma::quad(help_funct, 0, z) * comv_dist(c))
}

radius_of_curvature <- function(c) {
  if (c$type == "FlatLCDM") {
    k <- 0
    return(0.0)
  } else {
    if (c$type == "ClosedLCDM") {
      k <- 1
    } else {
      k <- -1
    }
    return(sqrt(-(k * 299792.458^2) / (c$curvature_crit * (100 * c$h_per_s)^2)))
  }
}

metric_distance <- function(c, z) {
  if (c$type == "FlatLCDM") {
    return(comoving_distance(c, z))
  } else if (c$type == "ClosedLCDM") {
    R0 <- radius_of_curvature(c)
    return(R0 * sin(comoving_distance(c, z) / R0))
  } else {
    R0 <- radius_of_curvature(c)
    return(R0 * sinh(comoving_distance(c, z) / R0))
  }
}

luminosity_distance <- function(c, z) {
  return((1 + z) * metric_distance(c, z))
}

angular_diameter_distance <- function(c, z) {
  return(metric_distance(c, z) / (1 + z))
}


a_of_t <- function(c, t) {
  f <- function(a) t_as_func_a_in_Year(c, a) - t # Root finding for t(a) = t
  a <- pracma::fzero(f, 1)$x # Numerical root finding starting from a=1
  return(a)
}

test_that("km_to_Mpc and Mpc_to_km are inverses", {
  expect_equal(km_to_Mpc(Mpc_to_km(1)), 1)
  expect_equal(Mpc_to_km(km_to_Mpc(3.086e19)), 3.086e19)
})

test_that("km_to_Mpc gives approximately 1 for 1 Mpc in km", {
  one_mpc_in_km <- 3.262e6 * 9.461e12
  expect_equal(km_to_Mpc(one_mpc_in_km), 1)
})

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/06_thermo2.R
\name{photon_energy_density_fn_T}
\alias{photon_energy_density_fn_T}
\title{Photon Energy Density as a Function of Temperature}
\usage{
photon_energy_density_fn_T(temp, unit = "eV")
}
\arguments{
\item{temp}{Numeric. Temperature in either eV or Kelvin.}

\item{unit}{Character. The unit of the temperature input. Either
\code{"eV"} (default) or \code{"K"}.}
}
\value{
Numeric. The photon energy density in natural units.
}
\description{
Calculates the photon energy density for a given temperature \eqn{T},
using \eqn{\rho_\gamma = \frac{\pi^2}{15} T^4}.
}
\examples{
photon_energy_density_fn_T(1, "eV")
photon_energy_density_fn_T(300, "K")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/06_thermo2.R
\name{photon_number_density_fn_z}
\alias{photon_number_density_fn_z}
\title{Photon Number Density as a Function of Redshift}
\usage{
photon_number_density_fn_z(z)
}
\arguments{
\item{z}{Numeric. The redshift value.}
}
\value{
Numeric. The photon number density in natural units.
}
\description{
Calculates the photon number density at a given cosmological redshift
\eqn{z} by first converting to temperature in eV.
}
\examples{
photon_number_density_fn_z(1300)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/01_angle.R
\name{dms_to_deg}
\alias{dms_to_deg}
\title{Convert Degrees, Minutes, and Seconds (DMS) to Decimal Degrees}
\usage{
dms_to_deg(d, m, s, digit = 5)
}
\arguments{
\item{d}{Numeric or character. The degrees component. If a single character
string is provided (e.g., \code{"12\u00B034'56\""}), the degrees, minutes,
and seconds are parsed automatically, and \code{m} and \code{s} should be
omitted.}

\item{m}{Numeric. The minutes component. Must be less than 60. Optional when
\code{d} is a string.}

\item{s}{Numeric. The seconds component. Must be less than 60. Optional when
\code{d} is a string.}

\item{digit}{Integer. Number of decimal places to round the result to.
Default is \code{5}.}
}
\value{
Numeric. The angle in decimal degrees.
}
\description{
Converts an angle expressed in Degrees-Minutes-Seconds (DMS) format to
decimal degrees. Accepts either separate numeric arguments or a single
formatted string.
}
\examples{
dms_to_deg(d = 12, m = 34, s = 56)
dms_to_deg(d = "12\u00B034'56\"", digit = 3)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/01_angle.R
\name{hms_to_deg}
\alias{hms_to_deg}
\title{Convert Hours, Minutes, and Seconds (HMS) to Decimal Degrees}
\usage{
hms_to_deg(h, m, s, digit = 5)
}
\arguments{
\item{h}{Numeric or character. The hours component. If a single character
string is provided (e.g., \code{"03h15m30s"}), the hours, minutes, and
seconds are parsed automatically, and \code{m} and \code{s} should be
omitted.}

\item{m}{Numeric. The minutes component. Optional when \code{h} is a
string.}

\item{s}{Numeric. The seconds component. Optional when \code{h} is a
string.}

\item{digit}{Integer. Number of decimal places to round the result to.
Default is \code{5}.}
}
\value{
Numeric. The angle in decimal degrees.
}
\description{
Converts an angle expressed in Hours-Minutes-Seconds (HMS) format to
decimal degrees. Accepts either separate numeric arguments or a single
formatted string.
}
\examples{
hms_to_deg(h = 3, m = 15, s = 30)
hms_to_deg(h = 3, m = 15, s = 30.123)
hms_to_deg(h = "03h15m30s")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/01_angle.R
\name{rad2deg}
\alias{rad2deg}
\title{Convert Radians to Degrees}
\usage{
rad2deg(rad)
}
\arguments{
\item{rad}{Numeric. The angle(s) in radians to convert to degrees.}
}
\value{
Numeric. The corresponding angle(s) in degrees.
}
\description{
Converts angles from radians to degrees. Supports vectorized input.
}
\examples{
rad2deg(pi)
rad2deg(c(0, pi / 2, pi))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/06_thermo2.R
\name{soln_saha}
\alias{soln_saha}
\title{Deviation of Ionization Fraction from 0.5}
\usage{
soln_saha(z)
}
\arguments{
\item{z}{Numeric. The redshift value.}
}
\value{
Numeric. \eqn{X_e - 0.5}.
}
\description{
Returns \eqn{X_e(z) - 0.5}, useful for finding the redshift at which the
ionization fraction equals 0.5 (e.g., via root-finding).
}
\examples{
soln_saha(1350)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/01_angle.R
\name{deg_to_hms}
\alias{deg_to_hms}
\title{Convert Decimal Degrees to Hours, Minutes, and Seconds (HMS) Format}
\usage{
deg_to_hms(deg, type = "cat", digit = 5)
}
\arguments{
\item{deg}{Numeric vector of angles in decimal degrees. Values can range
from 0 to 360.}

\item{type}{Character string specifying the output format. Options are:
\itemize{
  \item \code{"cat"} (default): Prints a formatted string
    (e.g., \code{3H0M0S}).
  \item \code{"mat"}: Returns a matrix with columns for hours, minutes,
    and seconds.
}}

\item{digit}{Integer specifying the number of decimal places to round
seconds to. Default is \code{5}.}
}
\value{
When \code{type = "cat"}, prints the HMS string and returns
  \code{NULL} invisibly. When \code{type = "mat"}, returns a numeric matrix
  with columns \code{HRS}, \code{MIN}, and \code{SEC}.
}
\description{
Converts a decimal degree angle to Hours-Minutes-Seconds (HMS) format,
which is used in astronomy to express Right Ascension (RA).
}
\examples{
deg_to_hms(deg = 45)
deg_to_hms(deg = 45, type = "mat")
deg_to_hms(deg = 177.74208, digit = 3)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/04_cosmology.R
\name{Mpc_to_km}
\alias{Mpc_to_km}
\title{Convert Megaparsecs to Kilometers}
\usage{
Mpc_to_km(mpc)
}
\arguments{
\item{mpc}{A numeric value representing the distance in megaparsecs.}
}
\value{
A numeric value representing the equivalent distance in kilometers (km).
}
\description{
This function converts a distance value from megaparsecs (Mpc) to kilometers (km).
The conversion factor is based on 1 parsec being equivalent to 3.262 light-years,
and 1 light-year being approximately \eqn{9.461 \times 10^{12}} kilometers.
}
\examples{
Mpc_to_km(1) # Converts 1 Mpc to kilometers
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/03_gaia.R
\name{get_gaia_data}
\alias{get_gaia_data}
\title{Query Gaia Archive Data}
\usage{
get_gaia_data(vars, condition)
}
\arguments{
\item{vars}{A character string specifying the variables (columns) to
retrieve, separated by commas (e.g., \code{"source_id, ra, dec"}).}

\item{condition}{A character string specifying the SQL WHERE clause used to
filter rows (e.g., \code{"parallax > 10"}).}
}
\value{
A data frame containing the requested columns for all rows matching
  \code{condition}.
}
\description{
Queries the Gaia Archive TAP service to retrieve stellar data based on
specified variables and filter conditions. Uses the Gaia Early Data Release
3 (EDR3) catalog.
}
\details{
This function sends a synchronous ADQL query to the Gaia Archive
  TAP service at \url{https://gea.esac.esa.int/tap-server/tap/sync}. An
  internet connection is required.
}
\examples{
\donttest{
vars <- "source_id, ra, dec, phot_bp_mean_mag, phot_rp_mean_mag, parallax"
condition <- "parallax > 40"
result <- get_gaia_data(vars, condition)
head(result)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/00_constants.R
\name{constant_value}
\alias{constant_value}
\title{Retrieve the Value of a Physical Constant}
\usage{
constant_value(constant_name, unit = "SI")
}
\arguments{
\item{constant_name}{Character. The name (or part of the name) of the
constant to search for. Case-insensitive and partial matches are allowed.}

\item{unit}{Character. The unit system to retrieve the constant in. Options
are:
\itemize{
  \item \code{"SI"}: Retrieves the value in SI units (default).
  \item \code{"Natural"}: Retrieves the value in natural units.
}}
}
\value{
A list containing:
  \itemize{
    \item \code{name}: The full name of the constant.
    \item \code{value}: The numeric value of the constant.
    \item \code{unit}: The unit string for the selected unit system.
  }
}
\description{
Retrieves the value and unit of a specified physical constant from the
\code{constants_df} dataset. You can search by the constant's name and
choose whether to retrieve the value in SI units or natural units.
}
\examples{
constant_value("speed of light", unit = "SI")
constant_value("planck", unit = "SI")
constant_value("electron mass", unit = "Natural")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/00_constants.R
\name{constants_df}
\alias{constants_df}
\title{Fundamental Physical Constants in SI and Natural Units}
\format{
A tibble with 19 rows and 6 columns:
\describe{
  \item{name}{Character. Name of the physical constant.}
  \item{symbol}{Character. Symbol representing the constant.}
  \item{value_SI}{Numeric. The value of the constant in SI units.}
  \item{unit_SI}{Character. The SI unit for the constant.}
  \item{value_Natural}{Numeric. The value of the constant in natural units.}
  \item{unit_Natural}{Character. The unit of the constant in natural units.}
}
}
\usage{
constants_df
}
\description{
A dataset containing commonly used physical constants in both SI units and
natural units. The dataset includes the constant's name, symbol, value in
SI units, SI unit, value in natural units, and natural unit representation.
}
\value{
A \code{tibble} (data frame) with 19 rows and 6 columns:
\describe{
  \item{name}{Character. Full name of the physical constant.}
  \item{symbol}{Character. Symbol representing the constant.}
  \item{value_SI}{Numeric. Value of the constant in SI units.}
  \item{unit_SI}{Character. The SI unit string for the constant.}
  \item{value_Natural}{Numeric. Value of the constant in natural units.}
  \item{unit_Natural}{Character. The natural unit string for the constant.}
}
}
\examples{
constants_df
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/01_angle.R
\name{deg2rad}
\alias{deg2rad}
\title{Convert Degrees to Radians}
\usage{
deg2rad(deg)
}
\arguments{
\item{deg}{Numeric. The angle(s) in degrees to convert to radians.}
}
\value{
Numeric. The corresponding angle(s) in radians.
}
\description{
Converts angles from degrees to radians. Supports vectorized input.
}
\examples{
deg2rad(180)
deg2rad(c(0, 90, 180))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/06_thermo2.R
\name{photon_number_density_fn_T}
\alias{photon_number_density_fn_T}
\title{Photon Number Density as a Function of Temperature}
\usage{
photon_number_density_fn_T(temp, unit = "eV")
}
\arguments{
\item{temp}{Numeric. Temperature in either eV or Kelvin.}

\item{unit}{Character. The unit of the temperature input. Either
\code{"eV"} (default) or \code{"K"}.}
}
\value{
Numeric. The photon number density in natural units.
}
\description{
Calculates the photon number density for a given temperature \eqn{T},
using \eqn{n_\gamma = \frac{2\zeta(3)}{\pi^2} T^3}.
}
\examples{
photon_number_density_fn_T(1, "eV")
photon_number_density_fn_T(300, "K")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/06_thermo2.R
\name{Saha_Xe}
\alias{Saha_Xe}
\title{Ionization Fraction Using the Saha Equation}
\usage{
Saha_Xe(z)
}
\arguments{
\item{z}{Numeric. The redshift value.}
}
\value{
Numeric. The ionization fraction \eqn{X_e} (between 0 and 1).
}
\description{
Calculates the ionization fraction \eqn{X_e} as a function of redshift
\eqn{z} using the Saha equation for hydrogen recombination.
}
\examples{
Saha_Xe(1300)
Saha_Xe(1100)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/04_cosmology.R
\name{km_to_Mpc}
\alias{km_to_Mpc}
\title{Convert Kilometers to Megaparsecs}
\usage{
km_to_Mpc(km)
}
\arguments{
\item{km}{A numeric value representing the distance in kilometers.}
}
\value{
A numeric value representing the equivalent distance in megaparsecs (Mpc).
}
\description{
This function converts a distance value from kilometers (km) to megaparsecs (Mpc).
The conversion factor is based on 1 parsec being equivalent to 3.262 light-years,
and 1 light-year being approximately \eqn{9.461 \times 10^{12}} kilometers.
}
\examples{
km_to_Mpc(3.086e19) # Converts 3.086e19 km (approx. 1 Mpc) to megaparsecs
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/01_angle.R
\name{deg_to_dms}
\alias{deg_to_dms}
\title{Convert Decimal Degrees to Degrees, Minutes, and Seconds (DMS) Format}
\usage{
deg_to_dms(deg, type = "cat", digit = 5)
}
\arguments{
\item{deg}{Numeric vector of angles in decimal degrees. All values must be
between -90 and 90.}

\item{type}{Character string specifying the output format. Options are:
\itemize{
  \item \code{"cat"} (default): Prints the angle as a formatted string
    in DMS notation.
  \item \code{"mat"}: Returns a matrix with columns for sign, degrees,
    minutes, and seconds.
}}

\item{digit}{Integer specifying the number of digits to round the seconds
to. Default is \code{5}.}
}
\value{
When \code{type = "cat"}, prints the DMS string and returns
  \code{NULL} invisibly. When \code{type = "mat"}, returns a character
  matrix with columns \code{SIGN}, \code{DEG}, \code{MIN}, and \code{SEC}.
}
\description{
Converts a decimal degree angle to the Degrees-Minutes-Seconds (DMS) format
commonly used in astronomy for expressing declination and other angles.
}
\examples{
deg_to_dms(45.5042)
deg_to_dms(-12.5, type = "mat")
deg_to_dms(c(10.25, 45.5), type = "mat")
}
